#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# This file is part of IcepapOCS link:
#        https://github.com/ALBA-Synchrotron/IcepapOCS
#
# Copyright 2017:
#       MAX IV Laboratory, Lund, Sweden
#       CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
#
# You should have received a copy of the GNU General Public License
# along with IcepapOCS. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------

from PyQt5 import QtWidgets, uic
import os
from pkg_resources import resource_filename


class DialogSettings(QtWidgets.QDialog):

    def __init__(self, parent, settings):
        QtWidgets.QDialog.__init__(self, parent)
        self.parent = parent
        ui_filename = resource_filename('icepaposc.ui',
                                        'dialog_settings.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.settings = settings
        self.apply_button = self.ui.bbApplyClose.button(
            QtWidgets.QDialogButtonBox.Apply)
        self.close_button = self.ui.bbApplyClose.button(
            QtWidgets.QDialogButtonBox.Close)
        self._connect_signals()
        self._update_gui_rate()
        self.ui.sbSampleRate.setMinimum(self.settings.sample_rate_min)
        self.ui.sbSampleRate.setMaximum(self.settings.sample_rate_max)
        self.ui.sbSampleRate.setValue(self.settings.sample_rate)
        self.ui.sbDumpRate.setMinimum(self.settings.dump_rate_min)
        self.ui.sbDumpRate.setMaximum(self.settings.dump_rate_max)
        self.ui.sbDumpRate.setValue(self.settings.dump_rate)
        self.ui.sbLenAxisX.setMinimum(self.settings.default_x_axis_len_min)
        self.ui.sbLenAxisX.setMaximum(self.settings.default_x_axis_len_max)
        self.ui.sbLenAxisX.setValue(self.settings.default_x_axis_len)
        self.ui.cbUseAutoSave.setChecked(self.settings.use_auto_save)
        self.ui.cbAppend.setChecked(self.settings.use_append)
        self.ui.sbAutoSaveInterval.setMinimum(self.settings.as_interval_min)
        self.ui.sbAutoSaveInterval.setMaximum(self.settings.as_interval_max)
        self.ui.sbAutoSaveInterval.setValue(self.settings.as_interval)
        self.ui.leDataFolder.setText(self.settings.as_folder)
        self._as_state_changed()
        self.apply_button.setDisabled(True)

    def _connect_signals(self):
        self.ui.sbSampleRate.valueChanged.connect(self._sample_rate_changed)
        self.ui.sbDumpRate.valueChanged.connect(self._dump_rate_changed)
        self.ui.sbLenAxisX.valueChanged.connect(self._x_axis_length_changed)
        self.ui.cbUseAutoSave.stateChanged.connect(self._as_state_changed)
        self.ui.cbAppend.stateChanged.connect(self._append_changed)
        self.ui.sbAutoSaveInterval.valueChanged.connect(self._as_intvl_changed)
        self.ui.btnOpenFolderDlg.clicked.connect(self._launch_folder_dialog)
        self.ui.leDataFolder.textChanged.connect(self._set_apply_state)
        self.apply_button.clicked.connect(self._apply)
        self.close_button.clicked.connect(self.close)

    def _sample_rate_changed(self):
        self._update_gui_rate()
        self._set_apply_state()

    def _dump_rate_changed(self):
        self._update_gui_rate()
        self._set_apply_state()

    def _x_axis_length_changed(self):
        self._set_apply_state()

    def _set_apply_state(self):
        eq = self.ui.sbSampleRate.value() == self.settings.sample_rate and \
           self.ui.sbDumpRate.value() == self.settings.dump_rate and \
           self.ui.sbLenAxisX.value() == self.settings.default_x_axis_len and \
           self.ui.cbUseAutoSave.isChecked() == \
           self.settings.use_auto_save and \
           self.ui.cbAppend.isChecked() == self.settings.use_append and \
           self.ui.sbAutoSaveInterval.value() == \
           self.settings.as_interval and \
           self.ui.leDataFolder.text() == self.settings.as_folder
        self.apply_button.setDisabled(eq)

    def _update_gui_rate(self):
        update_rate = self.ui.sbSampleRate.value() * self.ui.sbDumpRate.value()
        self.ui.leGuiUpdateRate.setText(str(update_rate))

    def _as_state_changed(self):
        use = self.ui.cbUseAutoSave.isChecked()
        self.ui.cbAppend.setEnabled(use)
        self.ui.sbAutoSaveInterval.setEnabled(use)
        self.ui.leDataFolder.setEnabled(use)
        self.ui.btnOpenFolderDlg.setEnabled(use)
        self._set_apply_state()

    def _append_changed(self):
        self._set_apply_state()

    def _as_intvl_changed(self):
        self._set_apply_state()

    def _launch_folder_dialog(self):
        folder_name = QtWidgets.QFileDialog.getExistingDirectory()
        if folder_name:
            self.ui.leDataFolder.setText(folder_name)
            self._set_apply_state()

    def _apply(self):
        auto_save_folder = self.ui.leDataFolder.text()
        if not self._is_valid_folder(auto_save_folder):
            return
        self.settings.sample_rate = self.ui.sbSampleRate.value()
        self.settings.dump_rate = self.ui.sbDumpRate.value()
        self.settings.default_x_axis_len = self.ui.sbLenAxisX.value()
        self.settings.use_auto_save = self.ui.cbUseAutoSave.isChecked()
        self.settings.use_append = self.ui.cbAppend.isChecked()
        self.settings.as_interval = self.ui.sbAutoSaveInterval.value()
        self.settings.as_folder = auto_save_folder
        self.settings.update()
        self.parent.settings_updated()
        self.apply_button.setDisabled(True)

    @staticmethod
    def _is_valid_folder(folder):
        # Make sure path exists.
        if not os.path.exists(folder):
            msg = 'Folder does not exist: {}\n'.format(folder)
            print(msg)
            QtWidgets.QMessageBox.critical(None, 'Set Data Folder', msg)
            return False
        # Create a dummy file name (hopefully unique for our test).
        fn = folder + '/IcePapOSCfolderTest.txt'
        # Create the dummy file for reading and writing.
        try:
            open(fn, "w+")
        except Exception as e:
            msg = 'Failed to create test file: {}\n{}'.format(fn, e)
            print(msg)
            QtWidgets.QMessageBox.critical(None, 'Bad Folder', msg)
            return False
        # Delete the dummy file.
        try:
            os.remove(fn)
        except OSError as e:
            msg = 'Failed to remove test file: {}\n{}'.format(fn, e)
            print(msg)
            QtWidgets.QMessageBox.critical(None, 'Remove Test File', msg)
            return False
        return True

    def done(self, r):
        """Overload of QDialog.done()."""
        self.parent.enable_action()
        QtWidgets.QDialog.done(self, r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# This file is part of IcepapOCS link:
#        https://github.com/ALBA-Synchrotron/IcepapOCS
#
# Copyright 2017:
#       MAX IV Laboratory, Lund, Sweden
#       CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
#
# You should have received a copy of the GNU General Public License
# along with IcepapOCS. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------

import os
from configparser import SafeConfigParser


class Settings:
    """Application settings."""

    def __init__(self):
        """Initializes an instance of class Settings."""
        user_path = os.path.expanduser("~")
        self.conf_file = os.path.join(user_path, ".icepaposc/settings.ini")
        if not os.path.exists(self.conf_file):
            self._create_file(user_path)

        # Settings for collector.
        self.sample_rate_min = 10  # [milliseconds]
        self.sample_rate_max = 1000  # [milliseconds]
        self.dump_rate_min = 1
        self.dump_rate_max = 100

        # Settings for GUI.
        self.default_x_axis_len_min = 5  # [Seconds]
        self.default_x_axis_len_max = 3600  # [Seconds]

        # Settings for auto save.
        self.as_interval_min = 1  # [Minutes]
        self.as_interval_max = 24 * 60  # [Minutes]

        self.sample_rate = 0
        self.dump_rate = 0
        self.default_x_axis_len = 0
        self.use_auto_save = False
        self.use_append = False
        self.as_interval = 5  # [Minutes]
        self.as_folder = user_path

        self._read_file()

    def _create_file(self, user_path):
        icepaosc_folder = os.path.join(user_path, '.icepaposc')
        if not os.path.exists(icepaosc_folder):
            os.mkdir(icepaosc_folder)
        conf = SafeConfigParser()
        conf.add_section('collector')
        conf.add_section('gui')
        conf.add_section('auto_save')
        conf.set('collector', 'tick_interval', '50')  # [milliseconds]
        conf.set('collector', 'sample_buf_len', '2')
        conf.set('gui', 'default_x_axis_len', '30')  # [Seconds]
        conf.set('auto_save', 'use', 'False')
        conf.set('auto_save', 'append', 'False')
        conf.set('auto_save', 'interval', '5')  # [Minutes]
        conf.set('auto_save', 'folder', user_path)
        with open(self.conf_file, 'w') as f:
            conf.write(f)

    def update(self):
        """Called when a setting has been changed."""
        conf = SafeConfigParser()
        conf.read(self.conf_file)
        conf.set('collector', 'tick_interval', str(self.sample_rate))
        conf.set('collector', 'sample_buf_len', str(self.dump_rate))
        conf.set('gui', 'default_x_axis_len', str(self.default_x_axis_len))
        conf.set('auto_save', 'use', str(self.use_auto_save))
        conf.set('auto_save', 'append', str(self.use_append))
        conf.set('auto_save', 'interval', str(self.as_interval))
        conf.set('auto_save', 'folder', str(self.as_folder))
        with open(self.conf_file, 'w') as f:
            conf.write(f)

    def _read_file(self):
        conf = SafeConfigParser()
        conf.read(self.conf_file)
        self.sample_rate = conf.getint('collector', 'tick_interval')
        self.dump_rate = conf.getint('collector', 'sample_buf_len')
        self.default_x_axis_len = conf.getint('gui', 'default_x_axis_len')
        self.use_auto_save = conf.getboolean('auto_save', 'use')
        self.use_append = conf.getboolean('auto_save', 'append')
        self.as_interval = conf.getint('auto_save', 'interval')
        self.as_folder = conf.get('auto_save', 'folder')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# This file is part of IcepapOCS link:
#        https://github.com/ALBA-Synchrotron/IcepapOCS
#
# Copyright 2017:
#       MAX IV Laboratory, Lund, Sweden
#       CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
#
# You should have received a copy of the GNU General Public License
# along with IcepapOCS. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------
import pyqtgraph as pg
import numpy as np
import collections
import time
import datetime

from PyQt5 import QtWidgets, Qt, QtCore, uic
from pkg_resources import resource_filename
from .collector import Collector
from .dialog_settings import DialogSettings
from .settings import Settings
from .axis_time import AxisTime
from .curve_item import CurveItem


class WindowMain(QtWidgets.QMainWindow):
    """A dialog for plotting IcePAP signals."""

    def __init__(self, host, port, timeout, siglist, selected_driver=None):
        """
        Initializes an instance of class WindowMain.

        host            - IcePAP system address.
        port            - IcePAP system port number.
        timeout         - Socket timeout.
        siglist         - List of predefined signals.
                            Element Syntax: <driver>:<signal name>:<Y-axis>
                            Example: ["1:PosAxis:1", "1:MeasI:2", "1:MeasVm:3"]
        selected_driver - The driver to display in combobox at startup.
        """
        QtWidgets.QMainWindow.__init__(self, None)
        ui_filename = resource_filename('icepaposc.ui', 'window_main.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)

        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.setWindowTitle('Oscilloscope  |  ' + host)
        self.settings = Settings()

        try:
            self.collector = Collector(host,
                                       port,
                                       timeout,
                                       self.settings,
                                       self.callback_collect)
        except Exception as e:
            msg = 'Failed to create main window.\n{}'.format(e)
            print(msg)
            QtWidgets.QMessageBox.critical(self, 'Create Main Window', msg)
            return

        self.subscriptions = {}
        self.curve_items = []
        self._paused = False

        # Set up the plot area.
        self.plot_widget = pg.PlotWidget()
        self._plot_item = self.plot_widget.getPlotItem()
        self.view_boxes = [self.plot_widget.getViewBox(),
                           pg.ViewBox(),
                           pg.ViewBox()]
        self.ui.vloCurves.setDirection(QtWidgets.QBoxLayout.BottomToTop)
        self.ui.vloCurves.addWidget(self.plot_widget)

        # Set up the X-axis.
        self._plot_item.getAxis('bottom').hide()  # Hide the original X-axis.
        self._axisTime = AxisTime(orientation='bottom')  # Create new X-axis.
        self._axisTime.linkToView(self.view_boxes[0])
        self._plot_item.layout.removeItem(self._plot_item.getAxis('bottom'))
        self._plot_item.layout.addItem(self._axisTime, 3, 1)
        self.now = self.collector.get_current_time()
        self.view_boxes[0].disableAutoRange(axis=self.view_boxes[0].XAxis)
        self.view_boxes[1].disableAutoRange(axis=self.view_boxes[1].XAxis)
        self.view_boxes[2].disableAutoRange(axis=self.view_boxes[2].XAxis)
        self._reset_x()

        # Set up the three Y-axes.
        self._plot_item.showAxis('right')
        self._plot_item.scene().addItem(self.view_boxes[1])
        self._plot_item.scene().addItem(self.view_boxes[2])
        ax3 = pg.AxisItem(orientation='right', linkView=self.view_boxes[2])
        self.axes = [self._plot_item.getAxis('left'),
                     self._plot_item.getAxis('right'), ax3]
        self.axes[1].linkToView(self.view_boxes[1])
        self.view_boxes[1].setXLink(self.view_boxes[0])
        self.view_boxes[2].setXLink(self.view_boxes[0])
        self._plot_item.layout.addItem(self.axes[2], 2, 3)
        self._plot_item.hideButtons()
        self._enable_auto_range_y()

        # Set up the crosshair vertical line.
        self.vertical_line = pg.InfiniteLine(angle=90, movable=False)
        self.view_boxes[0].addItem(self.vertical_line, ignoreBounds=True)

        # Initialize comboboxes and buttons.
        self._fill_combo_box_driver_ids(selected_driver)
        self._fill_combo_box_signals()
        self._select_axis_1()
        self._update_button_status()

        # Set up signalling connections.
        self._connect_signals()
        self.proxy = pg.SignalProxy(self.plot_widget.scene().sigMouseMoved,
                                    rateLimit=60,
                                    slot=self._mouse_moved)

        # Add any predefined signals.
        for sig in siglist:
            lst = sig.split(':')
            if len(lst) != 3:
                msg = 'Bad format of predefined signal "{}".\n' \
                      'It should be: ' \
                      '<driver>:<signal name>:<Y-axis>'.format(sig)
                print(msg)
                QtWidgets.QMessageBox.critical(self, 'Bad Signal Syntax', msg)
                return
            auto_save = True if sig == siglist[-1] else False
            self._add_signal(int(lst[0]), lst[1], int(lst[2]), auto_save)

        # Set up auto save of collected signal data.
        self._save_ticker = QtCore.QTimer()
        self._save_ticker.timeout.connect(self._auto_save)
        self._save_time = None
        self._idx = 0
        self._settings_updated = False
        self._file_path = None
        self._old_use_append = self.settings.use_append
        self._prepare_next_auto_save()

    def _fill_combo_box_driver_ids(self, selected_driver):
        driver_ids = self.collector.get_available_drivers()
        for driver_id in driver_ids:
            self.ui.cbDrivers.addItem(str(driver_id))
        start_index = 0
        if selected_driver is not None:
            start_index = self.ui.cbDrivers.findText(str(selected_driver))
        self.ui.cbDrivers.setCurrentIndex(start_index)

    def _fill_combo_box_signals(self):
        signals = self.collector.get_available_signals()
        num_colors = len(CurveItem.colors)
        if num_colors < len(signals):
            msg = 'Internal error!\nNew signals added.\nAdd ' \
                  'more colors and pens.'
            print(msg)
            QtWidgets.QMessageBox.warning(self, 'Available Signals', msg)
            for i in range(num_colors):
                self.ui.cbSignals.addItem(signals[i])
        else:
            for sig in signals:
                self.ui.cbSignals.addItem(sig)
        self.ui.cbSignals.setCurrentIndex(0)

    def _connect_signals(self):
        self.ui.rbAxis1.clicked.connect(self._select_axis_1)
        self.ui.rbAxis2.clicked.connect(self._select_axis_2)
        self.ui.rbAxis3.clicked.connect(self._select_axis_3)
        self.ui.btnAdd.clicked.connect(self._add_button_clicked)
        self.ui.btnShift.clicked.connect(self._shift_button_clicked)
        self.ui.btnRemoveSel.clicked.connect(self._remove_selected_signal)
        self.ui.btnRemoveAll.clicked.connect(self._remove_all_signals)
        self.ui.btnCLoop.clicked.connect(self._signals_closed_loop)
        self.ui.btnCurrents.clicked.connect(self._signals_currents)
        self.ui.btnTarget.clicked.connect(self._signals_target)
        self.ui.btnClear.clicked.connect(self._clear_all)
        self.ui.btnSeeAll.clicked.connect(self._view_all_data)
        self.ui.btnResetX.clicked.connect(self._reset_x)
        self.ui.btnResetY.clicked.connect(self._enable_auto_range_y)
        self.ui.btnPause.clicked.connect(self._pause_x_axis)
        self.ui.btnNow.clicked.connect(self._goto_now)
        self.ui.btnSave.clicked.connect(self._save_to_file)
        self.ui.actionSave_to_File.triggered.connect(self._save_to_file)
        self.ui.actionSettings.triggered.connect(self._display_settings_dlg)
        self.ui.actionExit.triggered.connect(self.close)
        self.ui.actionClosed_Loop.triggered.connect(self._signals_closed_loop)
        self.ui.actionCurrents.triggered.connect(self._signals_currents)
        self.ui.actionTarget.triggered.connect(self._signals_target)
        self.view_boxes[0].sigResized.connect(self._update_views)

    def closeEvent(self, event):
        """Overloads (QMainWindow) QWidget.closeEvent()."""
        self._remove_all_signals()
        event.accept()

    def _update_views(self):
        """Updates the geometry of the view boxes."""
        self.view_boxes[1].setGeometry(self.view_boxes[0].sceneBoundingRect())
        self.view_boxes[2].setGeometry(self.view_boxes[0].sceneBoundingRect())
        self.view_boxes[1].linkedViewChanged(self.view_boxes[0],
                                             self.view_boxes[1].XAxis)
        self.view_boxes[2].linkedViewChanged(self.view_boxes[0],
                                             self.view_boxes[2].XAxis)

    def _update_button_status(self):
        val = self.ui.lvActiveSig.count() == 0
        self.ui.btnShift.setDisabled(val)
        self.ui.btnRemoveSel.setDisabled(val)
        self.ui.btnRemoveAll.setDisabled(val)

    def _update_plot_axes_labels(self):
        txt = ['', '', '']
        for ci in self.curve_items:
            t = "<span style='font-size: 8pt; " \
                "color: {};'>{}</span>".format(ci.color.name(), ci.signature)
            txt[ci.y_axis - 1] += t
        for i in range(0, len(self.axes)):
            self.axes[i].setLabel(txt[i])

    def _select_axis_1(self):
        self.ui.rbAxis1.setChecked(True)
        self.ui.rbAxis2.setChecked(False)
        self.ui.rbAxis3.setChecked(False)

    def _select_axis_2(self):
        self.ui.rbAxis1.setChecked(False)
        self.ui.rbAxis2.setChecked(True)
        self.ui.rbAxis3.setChecked(False)

    def _select_axis_3(self):
        self.ui.rbAxis1.setChecked(False)
        self.ui.rbAxis2.setChecked(False)
        self.ui.rbAxis3.setChecked(True)

    def _add_button_clicked(self):
        addr = int(self.ui.cbDrivers.currentText())
        my_signal_name = str(self.ui.cbSignals.currentText())
        my_axis = 1
        if self.ui.rbAxis2.isChecked():
            my_axis = 2
        elif self.ui.rbAxis3.isChecked():
            my_axis = 3
        self._add_signal(addr, my_signal_name, my_axis, True)

    def _add_signal(self, driver_addr, signal_name, y_axis, auto_save=False):
        """
        Adds a new curve to the plot area.

        driver_addr - IcePAP driver address.
        signal_name - Signal name.
        y_axis      - Y axis to plot against.
        """
        try:
            subscription_id = self.collector.subscribe(driver_addr,
                                                       signal_name)
        except Exception as e:
            msg = 'Failed to subscribe to signal {} ' \
                  'from driver {}.\n{}'.format(signal_name, driver_addr, e)
            print(msg)
            QtWidgets.QMessageBox.critical(self, 'Add Curve', msg)
            return
        try:
            color_idx = self.collector.get_signal_index(signal_name)
        except ValueError as e:
            msg = 'Internal error. Failed to retrieve index ' \
                  'for signal {}.\n{}'.format(signal_name, e)
            print(msg)
            QtWidgets.QMessageBox.critical(self, 'Add Curve', msg)
            return
        ci = CurveItem(subscription_id, driver_addr, signal_name,
                       y_axis, color_idx)
        self._add_curve(ci)
        self.curve_items.append(ci)
        self.collector.start(subscription_id)
        self.ui.lvActiveSig.addItem(ci.signature)
        index = len(self.curve_items) - 1
        self.ui.lvActiveSig.setCurrentRow(index)
        self.ui.lvActiveSig.item(index).setForeground(ci.color)
        self.ui.lvActiveSig.item(index).setBackground(Qt.QColor(0, 0, 0))
        self._update_plot_axes_labels()
        self._update_button_status()
        if auto_save:
            self._auto_save(True)

    def _remove_selected_signal(self):
        self._auto_save(True)
        index = self.ui.lvActiveSig.currentRow()
        ci = self.curve_items[index]
        self.collector.unsubscribe(ci.subscription_id)
        self._remove_curve_plot(ci)
        self.ui.lvActiveSig.takeItem(index)
        self.curve_items.remove(ci)
        self._update_plot_axes_labels()
        self._update_button_status()

    def _remove_all_signals(self):
        """Removes all signals."""
        self._auto_save(True)
        for ci in self.curve_items:
            self.collector.unsubscribe(ci.subscription_id)
            self._remove_curve_plot(ci)
        self.ui.lvActiveSig.clear()
        self.curve_items = []
        self._update_plot_axes_labels()
        self._update_button_status()

    def _shift_button_clicked(self):
        """Assign a curve to a different y axis."""
        index = self.ui.lvActiveSig.currentRow()
        ci = self.curve_items[index]
        self._remove_curve_plot(ci)
        ci.y_axis = (ci.y_axis % 3) + 1
        ci.update_signature()
        self._add_curve(ci)
        self.ui.lvActiveSig.takeItem(index)
        self.ui.lvActiveSig.insertItem(index, ci.signature)
        self.ui.lvActiveSig.item(index).setForeground(ci.color)
        self.ui.lvActiveSig.item(index).setBackground(Qt.QColor(0, 0, 0))
        self.ui.lvActiveSig.setCurrentRow(index)
        self._update_plot_axes_labels()

    def _add_curve(self, ci):
        """
        Create a new curve and add it to a viewbox.

        ci - Curve item that will be the owner.
        """
        my_curve = ci.create_curve()
        self.view_boxes[ci.y_axis - 1].addItem(my_curve)

    def _mouse_moved(self, evt):
        """
        Acts om mouse move.

        evt - Event containing the position of the mouse pointer.
        """
        pos = evt[0]  # The signal proxy turns original arguments into a tuple.
        if self.plot_widget.sceneBoundingRect().contains(pos):
            mouse_point = self.view_boxes[0].mapSceneToView(pos)
            time_value = mouse_point.x()
            try:
                date = datetime.datetime.fromtimestamp(time_value)
                pretty_time = date.strftime("%H:%M:%S.%f")[:-3]
            except ValueError:  # Time out of range.
                return
            txtmax = ''
            txtnow = ''
            txtmin = ''
            text_size = 10
            for ci in self.curve_items:
                tmp = "<span style='font-size: {}pt; color: {};'>|"
                tmp = tmp.format(text_size, ci.color.name())
                if ci.in_range(time_value):
                    txtmax += "{}{}</span>".format(tmp, ci.val_max)
                    txtnow += "{}{}</span>".format(tmp, ci.get_y(time_value))
                    txtmin += "{}{}</span>".format(tmp, ci.val_min)
            tmp = "|<span style='font-size: {}pt; color: white;'>{}</span>"
            txtnow += tmp.format(text_size, pretty_time)
            title = "<br>{}<br>{}<br>{}".format(txtmax, txtnow, txtmin)
            self.plot_widget.setTitle(title)
            self.vertical_line.setPos(mouse_point.x())

    def _remove_curve_plot(self, ci):
        """
        Remove a curve from the plot area.

        ci - Curve item to remove.
        """
        self.view_boxes[ci.y_axis - 1].removeItem(ci.curve)

    def _signals_closed_loop(self):
        """Display a specific set of curves."""
        self._remove_all_signals()
        drv_addr = int(self.ui.cbDrivers.currentText())
        self._add_signal(drv_addr, 'PosAxis', 1)
        self._add_signal(drv_addr, 'DifAxTgtenc', 2)
        self._add_signal(drv_addr, 'DifAxMotor', 2)
        self._add_signal(drv_addr, 'StatReady', 3)
        self._add_signal(drv_addr, 'StatMoving', 3)
        self._add_signal(drv_addr, 'StatSettling', 3)

        self._add_signal(drv_addr, 'StatOutofwin', 3, True)

        # self._add_signal(drv_addr, 'StatOutofwin', 3)
        # self._add_signal(drv_addr, 'StatWarning', 3)
        # self.view_boxes[0].enableAutoRange(axis=self.view_boxes[0].YAxis)
        # self.view_boxes[1].disableAutoRange(axis=self.view_boxes[1].YAxis)
        # self.view_boxes[1].setYRange(-30, 70, padding=0)
        # self.view_boxes[2].disableAutoRange(axis=self.view_boxes[2].YAxis)
        # self.view_boxes[2].setYRange(-1, 20, padding=0)

    def _signals_currents(self):
        """Display a specific set of curves."""
        self._remove_all_signals()
        drv_addr = int(self.ui.cbDrivers.currentText())
        self._add_signal(drv_addr, 'PosAxis', 1)
        self._add_signal(drv_addr, 'MeasI', 2)

        self._add_signal(drv_addr, 'MeasVm', 3, True)
        # self._add_signal(drv_addr, 'MeasVm', 3)
        # # Ajust plot axis
        # self.view_boxes[0].enableAutoRange(axis=self.view_boxes[0].YAxis)
        # self.view_boxes[1].disableAutoRange(axis=self.view_boxes[1].YAxis)
        # self.view_boxes[1].setYRange(-9, 10, padding=0)
        # self.view_boxes[2].enableAutoRange(axis=self.view_boxes[2].YAxis)

    def _signals_target(self):
        """Display a specific set of curves."""
        self._remove_all_signals()
        drv_addr = int(self.ui.cbDrivers.currentText())
        self._add_signal(drv_addr, 'PosAxis', 1)
        self._add_signal(drv_addr, 'EncTgtenc', 2, True)

    def _clear_all(self):
        """Clear all the displayed curves."""
        self._auto_save()
        for ci in self.curve_items:
            ci.clear()

    def _view_all_data(self):
        """Adjust X axis to view all collected data."""
        time_start = self.collector.get_current_time()
        for ci in self.curve_items:
            t = ci.start_time()
            if 0 < t < time_start:
                time_start = t
        self.view_boxes[0].setXRange(time_start,
                                     self.collector.get_current_time(),
                                     padding=0)

    def _reset_x(self):
        """
        Reset the length of the X axis to
        the initial number of seconds (setting).
        """
        now = self.collector.get_current_time()
        start = now - self.settings.default_x_axis_len
        self.view_boxes[0].setXRange(start, now, padding=0)

    def _enable_auto_range_y(self):
        self.view_boxes[0].enableAutoRange(axis=self.view_boxes[0].YAxis)
        self.view_boxes[1].enableAutoRange(axis=self.view_boxes[1].YAxis)
        self.view_boxes[2].enableAutoRange(axis=self.view_boxes[2].YAxis)

    def _pause_x_axis(self):
        """Freeze the X axis."""
        if self._paused:
            self._paused = False
            self.ui.btnPause.setText('Pause')
        else:
            self._paused = True
            self.ui.btnPause.setText('Run')
        self.ui.btnClear.setDisabled(self._paused)

    def _goto_now(self):
        """Pan X axis to display newest values."""
        now = self.collector.get_current_time()
        x_min = self.view_boxes[0].viewRange()[0][0]
        x_max = self.view_boxes[0].viewRange()[0][1]
        self.view_boxes[0].setXRange(now - (x_max - x_min), now, padding=0)

    def enable_action(self, enable=True):
        """Enables or disables menu item File|Settings."""
        self.ui.actionSettings.setEnabled(enable)

    def _save_to_file(self):
        if not self.curve_items:
            return
        capt = "Save to csv file"
        fn = QtWidgets.QFileDialog.getSaveFileName(caption=capt,
                                                   filter="*.csv")
        if not fn:
            return
        if fn[-4:] != ".csv":
            fn = fn + ".csv"
        try:
            f = open(fn, "w+")
        except Exception as e:
            msg = 'Failed to open/create file: {}\n{}'.format(fn, e)
            print(msg)
            QtWidgets.QMessageBox.critical(self, 'File Open Failed', msg)
            return
        self._create_csv_file(f)
        f.close()

    def _create_csv_file(self, csv_file):
        my_dict = collections.OrderedDict()
        for ci in self.curve_items:
            header = "time-{}-{}".format(ci.driver_addr, ci.signal_name)
            my_dict[header] = ci.array_time
            header = "val-{}-{}".format(ci.driver_addr, ci.signal_name)
            my_dict[header] = ci.array_val
        key_longest = list(my_dict.keys())[0]
        for key in my_dict:
            if my_dict[key][0] < my_dict[key_longest][0]:
                key_longest = key
        for key in my_dict:
            delta = len(my_dict[key_longest]) - len(my_dict[key])
            my_dict[key] = delta * [np.nan] + my_dict[key]
        for key in my_dict:
            csv_file.write(",{}".format(key))
        csv_file.write("\n")
        for idx in range(0, len(my_dict[key_longest])):
            line = str(idx)
            for key in my_dict:
                line += ",{}".format(my_dict[key][idx])
            csv_file.write(line + '\n')

    def _auto_save(self, use_new_file=False):
        if not self.curve_items or not self._file_path:
            return
        if not self._settings_updated and not self.settings.use_auto_save:
            return
        self._save_ticker.stop()

        # Create matrix.
        my_dict = collections.OrderedDict()
        for ci in self.curve_items:
            start_idx = ci.get_time_index(self._save_time)
            header = "time-{}-{}".format(ci.driver_addr, ci.signal_name)
            my_dict[header] = ci.array_time[start_idx:]
            header = "val-{}-{}".format(ci.driver_addr, ci.signal_name)
            my_dict[header] = ci.array_val[start_idx:]
        key_longest = None
        for key in my_dict:  # Find a non empty list.
            if my_dict[key]:
                key_longest = key
                break
        if not key_longest:
            self._prepare_next_auto_save(True)
            return
        for key in my_dict:  # Find the longest list.
            if my_dict[key] and my_dict[key][0] < my_dict[key_longest][0]:
                key_longest = key
        for key in my_dict:  # Fill up the shorter lists with nan.
            delta = len(my_dict[key_longest]) - len(my_dict[key])
            my_dict[key] = delta * [np.nan] + my_dict[key]

        # Write matrix to file.
        try:
            f = open(self._file_path, self._get_write_mode())
        except Exception as e:
            msg = 'Failed to open file: {}\n{}'.format(self._file_path, e)
            print(msg)
            QtWidgets.QMessageBox.critical(self, 'File Open Failed', msg)
            return
        if self._idx == 0:
            for key in my_dict:
                f.write(",{}".format(key))
            f.write("\n")
        for i in range(0, len(my_dict[key_longest])):
            line = str(self._idx)
            self._idx += 1
            for key in my_dict:
                line += ",{}".format(my_dict[key][i])
            f.write(line + '\n')
        f.close()

        self._prepare_next_auto_save(use_new_file)

    def _prepare_next_auto_save(self, use_new_file=False):
        if self.settings.use_auto_save:
            if use_new_file or not self.settings.use_append or \
                    not self._file_path or self._settings_updated:
                self._set_new_file_path()
            self._save_time = time.time()
            self._save_ticker.start(60000 * self.settings.as_interval)
        else:
            self._save_time = None
            self._file_path = None

    def _set_new_file_path(self):
        self._idx = 0
        time_str = time.strftime("%Y%m%d_%H%M%S", time.localtime())
        file_name = "IcepapOSC_{}.csv".format(time_str)
        self._file_path = self.settings.as_folder + '/' + file_name

    def _get_write_mode(self):
        do_append = self.settings.use_append
        if self._settings_updated:
            do_append = self._old_use_append
        return "a+" if do_append else "w+"

    def _display_settings_dlg(self):
        self.enable_action(False)
        dlg = DialogSettings(self, self.settings)
        dlg.show()

    def settings_updated(self):
        """Settings have been changed."""
        self._settings_updated = True
        if self._file_path:
            self._auto_save(True)
        else:
            self._prepare_next_auto_save()
        self._old_use_append = self.settings.use_append
        self._settings_updated = False
        self._reset_x()

    def callback_collect(self, subscription_id, value_list):
        """
        Callback function that stores the data collected from IcePAP.

        subscription_id - Subscription id.
        value_list - List of tuples (time, value).
        """
        for ci in self.curve_items:
            if ci.subscription_id == subscription_id:
                ci.collect(value_list)
        if not self._paused:
            self._update_view()

    def _update_view(self):
        x_min = self.view_boxes[0].viewRange()[0][0]
        x_max = self.view_boxes[0].viewRange()[0][1]

        # Update the X-axis.
        now_in_range = self.now <= x_max
        self.now = self.collector.get_current_time()
        if now_in_range:
            self.view_boxes[0].setXRange(self.now - (x_max - x_min),
                                         self.now,
                                         padding=0)
        self.ui.btnNow.setDisabled(now_in_range)

        # Update the curves.
        for ci in self.curve_items:
            ci.update_curve(x_min, x_max)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# This file is part of IcepapOCS link:
#        https://github.com/ALBA-Synchrotron/IcepapOCS
#
# Copyright 2017:
#       MAX IV Laboratory, Lund, Sweden
#       CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
#
# You should have received a copy of the GNU General Public License
# along with IcepapOCS. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------


class Channel:
    """A container class holding information for a driver signal."""

    def __init__(self, address, signal_name):
        """
        Initializes an instance of class Channel.

        address     - IcePAP driver id.
        signal_name - Signal name.
        """
        self.icepap_address = address
        self.sig_name = signal_name
        self.measure_resolution = 1.
        self.collected_samples = []

    def equals(self, icepap_addr, signal_name):
        """
        Checks for equality.

        icepap_addr - IcePAP address.
        signal_name - Signal name.
        Return: True if equal. False otherwise.
        """
        if icepap_addr != self.icepap_address:
            return False
        if signal_name != self.sig_name:
            return False
        return True

    def set_measure_resolution(self, cfg):
        """
        Set measure resolution.

        cfg - Config info extracted from IcePAP.
        """
        tgtenc = cfg['TGTENC'].upper()
        shftenc = cfg['SHFTENC'].upper()
        axisnstep = cfg['ANSTEP']
        axisnturn = cfg['ANTURN']
        nstep = axisnstep
        nturn = axisnturn
        if tgtenc == 'ABSENC' or (tgtenc == 'NONE' and shftenc == 'ABSENC'):
            nstep = cfg['ABSNSTEP']
            nturn = cfg['ABSNTURN']
        elif tgtenc == 'ENCIN' or (tgtenc == 'NONE' and shftenc == 'ENCIN'):
            nstep = cfg['EINNSTEP']
            nturn = cfg['EINNTURN']
        elif tgtenc == 'INPOS' or (tgtenc == 'NONE' and shftenc == 'INPOS'):
            nstep = cfg['INPNSTEP']
            nturn = cfg['INPNTURN']
        self.measure_resolution = (float(nstep) / float(nturn)) / \
                                  (float(axisnstep) / float(axisnturn))



from setuptools import setup
from setuptools import find_packages

# The version is updated automatically with bumpversion
# Do not update manually
__version = '0.5.2'

# windows installer:
# python setup.py bdist_wininst


long_description = """ Python application to monitor and tune IcePAP based 
systems. """


setup(
    name="icepaposc",
    description="Python IcePAP Extension",
    version=__version,
    author="Jarkko Inki and Roberto Homs-Puron",
    author_email="ctbeamlines@cells.es",
    url="https://github.com/ALBA-Synchrotron/IcepapOCS",
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.5',
    keywords='APP',
    license="GPL",
    long_description=long_description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3.5',
        'Topic :: Communications',
        'Topic :: Software Development :: Libraries',
    ],
    entry_points={
        'console_scripts': [
            'icepaposc = icepaposc.__main__:main',
        ],
    },
    install_requires=[
        "PyQt5",
        "icepap",
        'pyqtgraph',
        'numpy',

    ],
    package_data={'': ['*.ui', ]}
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# This file is part of IcepapOCS link:
#        https://github.com/ALBA-Synchrotron/IcepapOCS
#
# Copyright 2017:
#       MAX IV Laboratory, Lund, Sweden
#       CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
#
# You should have received a copy of the GNU General Public License
# along with IcepapOCS. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------

from PyQt5 import QtCore
from collections import OrderedDict
from icepap import IcePAPController
from .channel import Channel
import time


class Collector:
    """Feeds a subscriber with collected IcePAP signal data."""

    def __init__(self, host, port, timeout, settings, callback):
        """
        Initializes an instance of class Collector.

        host     - The IcePAP system host name.
        port     - The IcePAP system port number.
        timeout  - Socket timeout.
        callback - A callback function used for sending collected signal
                   data back to the caller.
                   cb_func(subscription_id, value_list)
                       subscription_id - The subscription id retained when
                                         subscribing for a signal.
                       value_list      - A list of tuples
                                         (time_stamp, signal_value)
        """
        self.sig_getters = OrderedDict(
            [('PosAxis', self._getter_pos_axis),
             ('PosTgtenc', self._getter_pos_tgtenc),
             ('PosShftenc', self._getter_pos_shftenc),
             ('PosEncin', self._getter_pos_encin),
             ('PosAbsenc', self._getter_pos_absenc),
             ('PosInpos', self._getter_pos_inpos),
             ('PosMotor', self._getter_pos_motor),
             ('PosCtrlenc', self._getter_pos_ctrlenc),
             ('PosMeasure', self._getter_pos_measure),
             ('DifAxMeasure', self._getter_dif_ax_measure),
             ('DifAxMotor', self._getter_dif_ax_motor),
             ('DifAxTgtenc', self._getter_dif_ax_tgtenc),
             ('DifAxShftenc', self._getter_dif_ax_shftenc),
             ('DifAxCtrlenc', self._getter_dif_ax_ctrlenc),
             ('EncEncin', self._getter_enc_encin),
             ('EncAbsenc', self._getter_enc_absenc),
             ('EncTgtenc', self._getter_enc_tgtenc),
             ('EncInpos', self._getter_enc_inpos),
             ('StatReady', self._getter_stat_ready),
             ('StatMoving', self._getter_stat_moving),
             ('StatSettling', self._getter_stat_settling),
             ('StatOutofwin', self._getter_stat_outofwin),
             ('StatStopcode', self._getter_stat_stopcode),
             ('StatWarning', self._getter_stat_warning),
             ('StatLim+', self._getter_stat_limit_positive),
             ('StatLim-', self._getter_stat_limit_negative),
             ('StatHome', self._getter_stat_home),
             ('MeasI', self._getter_meas_i),
             ('MeasIa', self._getter_meas_ia),
             ('MeasIb', self._getter_meas_ib),
             ('MeasVm', self._getter_meas_vm)]
        )
        self.host = host
        self.port = port
        self.settings = settings
        self.cb = callback
        self.icepap_system = None
        self.channels_subscribed = {}
        self.channels = {}
        self.channel_id = 0
        self.current_channel = 0
        self.sig_list = list(self.sig_getters.keys())

        try:
            self.icepap_system = IcePAPController(self.host, self.port,
                                                  timeout, auto_axes=True)
        except Exception as e:
            msg = 'Failed to instantiate master controller.\nHost: ' \
                  '{}\nPort: {}\n{}'.format(self.host, self.port, e)
            raise Exception(msg)
        if not self.icepap_system:
            msg = 'IcePAP system {} has no active drivers! ' \
                  'Aborting.'.format(self.host)
            raise Exception(msg)

        self.ticker = QtCore.QTimer()
        self.ticker.timeout.connect(self._tick)
        self.ticker.start(self.settings.sample_rate)

    def get_available_drivers(self):
        """
        Retrieves the available drivers.

        Return: List of available drivers.
        """
        return self.icepap_system.axes

    def get_available_signals(self):
        """
        Retrieves the available signals.

        Return: List of available signals.
        """
        return self.sig_list

    def get_signal_index(self, signal_name):
        """
        Retrieves the fixed index of a signal from its name.

        Return: Signal index.
        """
        return self.sig_list.index(signal_name)

    @staticmethod
    def get_current_time():
        """
        Retrieves the current time.

        Return: Current time as seconds (with fractions) from 1970.
        """
        return time.time()

    def subscribe(self, icepap_addr, signal_name):
        """
        Creates a new subscription for signal values.

        icepap_addr - IcePAP driver number.
        signal_name - Signal name.
        Return - A positive integer id used when unsubscribing.
        """
        for ch in list(self.channels_subscribed.values()):
            if ch.equals(icepap_addr, signal_name):
                msg = 'Channel already exists.\nAddr: ' \
                      '{}\nSignal: {}'.format(icepap_addr, signal_name)
                raise Exception(msg)
        channel = Channel(icepap_addr, signal_name)
        sn = str(signal_name)
        cond_1 = sn.endswith('Tgtenc')
        cond_2 = sn.endswith('Shftenc')
        cond_3 = sn == 'DifAxMeasure'
        if cond_1 or cond_2 or cond_3:
            try:
                cfg = self.icepap_system[icepap_addr].get_cfg()
            except RuntimeError as e:
                msg = 'Failed to retrieve configuration parameters ' \
                      'for driver {}\n{}.'.format(icepap_addr, e)
                raise Exception(msg)
            if (cond_1 and cfg['TGTENC'].upper() == 'NONE') or \
                    (cond_2 and cfg['SHFTENC'].upper() == 'NONE'):
                msg = 'Signal {} is not mapped/valid.'.format(sn)
                raise Exception(msg)
            if cond_3:
                channel.set_measure_resolution(cfg)
        self.channel_id += 1
        self.channels_subscribed[self.channel_id] = channel
        return self.channel_id

    def start(self, subscription_id):
        """
        Starts collecting data for a subscription.

        subscription_id - The given subscription id.
        """
        if subscription_id in list(self.channels_subscribed.keys()) and \
                subscription_id not in list(self.channels.keys()):
            self.channels[subscription_id] = \
                self.channels_subscribed[subscription_id]

    def unsubscribe(self, subscription_id):
        """
        Cancels a subscription.

        subscription_id - The given subscription id.
        """
        if subscription_id in list(self.channels_subscribed.keys()):
            del self.channels[subscription_id]
            del self.channels_subscribed[subscription_id]

    def _tick(self):
        for subscription_id, channel in self.channels.items():
            self.current_channel = subscription_id
            try:
                addr = channel.icepap_address
                val = self.sig_getters[channel.sig_name](addr)
            except RuntimeError as e:
                msg = 'Failed to collect data for signal ' \
                      '{}\n{}'.format(channel.sig_name, e)
                print(msg)
                continue
            tv = (time.time(), val)
            channel.collected_samples.append(tv)
            if len(channel.collected_samples) >= self.settings.dump_rate:
                self.cb(subscription_id, channel.collected_samples)
                channel.collected_samples = []
        self.ticker.start(self.settings.sample_rate)

    def _getter_pos_axis(self, addr):
        return self.icepap_system[addr].pos

    def _getter_pos_tgtenc(self, addr):
        return self.icepap_system[addr].pos_tgtenc

    def _getter_pos_shftenc(self, addr):
        return self.icepap_system[addr].pos_shftenc

    def _getter_pos_encin(self, addr):
        return self.icepap_system[addr].pos_encin

    def _getter_pos_absenc(self, addr):
        return self.icepap_system[addr].pos_absenc

    def _getter_pos_inpos(self, addr):
        return self.icepap_system[addr].pos_inpos

    def _getter_pos_motor(self, addr):
        return self.icepap_system[addr].pos_motor

    def _getter_pos_ctrlenc(self, addr):
        return self.icepap_system[addr].pos_ctrlenc

    def _getter_pos_measure(self, addr):
        return self.icepap_system.get_fpos(self.icepap_system[addr].addr,
                                           'MEASURE')[0]

    def _getter_dif_ax_measure(self, addr):
        pos_measure = self._getter_pos_measure(addr) / \
                      self.channels[self.current_channel].measure_resolution
        return self._getter_pos_axis(addr) - pos_measure

    def _getter_dif_ax_motor(self, addr):
        return self._getter_pos_axis(addr) - self._getter_pos_motor(addr)

    def _getter_dif_ax_tgtenc(self, addr):
        return self._getter_pos_axis(addr) - self._getter_pos_tgtenc(addr)

    def _getter_dif_ax_shftenc(self, addr):
        return self._getter_pos_axis(addr) - self._getter_pos_shftenc(addr)

    def _getter_dif_ax_ctrlenc(self, addr):
        return self._getter_pos_axis(addr) - self._getter_pos_ctrlenc(addr)

    def _getter_enc_encin(self, addr):
        return self.icepap_system[addr].enc_encin

    def _getter_enc_absenc(self, addr):
        return self.icepap_system[addr].enc_absenc

    def _getter_enc_tgtenc(self, addr):
        return self.icepap_system[addr].enc_tgtenc

    def _getter_enc_inpos(self, addr):
        return self.icepap_system[addr].enc_inpos

    def _getter_stat_ready(self, addr):
        return 1 if self.icepap_system[addr].state_ready else 0

    def _getter_stat_moving(self, addr):
        return 1 if self.icepap_system[addr].state_moving else 0

    def _getter_stat_settling(self, addr):
        return 1 if self.icepap_system[addr].state_settling else 0

    def _getter_stat_outofwin(self, addr):
        return 1 if self.icepap_system[addr].state_outofwin else 0

    def _getter_stat_stopcode(self, addr):
        return self.icepap_system[addr].state_stop_code

    def _getter_stat_warning(self, addr):
        return 1 if self.icepap_system[addr].state_warning else 0

    def _getter_stat_limit_positive(self, addr):
        return 1 if self.icepap_system[addr].state_limit_positive else 0

    def _getter_stat_limit_negative(self, addr):
        return 1 if self.icepap_system[addr].state_limit_negative else 0

    def _getter_stat_home(self, addr):
        return 1 if self.icepap_system[addr].state_inhome else 0

    def _getter_meas_i(self, addr):
        return self.icepap_system[addr].meas_i

    def _getter_meas_ia(self, addr):
        return self.icepap_system[addr].meas_ia

    def _getter_meas_ib(self, addr):
        return self.icepap_system[addr].meas_ib

    def _getter_meas_vm(self, addr):
        return self.icepap_system[addr].meas_vm



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# This file is part of IcepapOCS link:
#        https://github.com/ALBA-Synchrotron/IcepapOCS
#
# Copyright 2017:
#       MAX IV Laboratory, Lund, Sweden
#       CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
#
# You should have received a copy of the GNU General Public License
# along with IcepapOCS. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------

from PyQt5 import QtCore, QtGui
from collections import namedtuple
from threading import RLock
from pyqtgraph import PlotCurveItem


class CurveItem:
    """Represents a curve to be plotted in a diagram."""

    SignalAppearance = namedtuple('SignalAppearance',
                                  ['pen_color', 'pen_width', 'pen_style'])

    colors = [
        SignalAppearance(QtGui.QColor(255, 255, 0), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(255, 0, 0), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(0, 255, 0), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(255, 255, 255), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(51, 153, 255), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(0, 255, 255), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(255, 0, 255), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(204, 153, 102), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(0, 0, 255), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(0, 255, 0), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(255, 204, 0), 1,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(153, 255, 153), 2,
                         QtCore.Qt.DotLine),
        SignalAppearance(QtGui.QColor(255, 170, 0), 2,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 0, 0), 2,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(0, 255, 255), 1,
                         QtCore.Qt.DotLine),
        SignalAppearance(QtGui.QColor(255, 170, 255), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(127, 255, 127), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 255, 127), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 0, 0), 2,
                         QtCore.Qt.DotLine),
        SignalAppearance(QtGui.QColor(255, 0, 0), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(0, 255, 0), 2,
                         QtCore.Qt.DotLine),
        SignalAppearance(QtGui.QColor(255, 255, 255), 2,
                         QtCore.Qt.SolidLine),
        SignalAppearance(QtGui.QColor(51, 153, 255), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 0, 255), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 153, 204), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(204, 153, 102), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 204, 0), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 0, 255), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 153, 204), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(204, 153, 102), 1,
                         QtCore.Qt.DashLine),
        SignalAppearance(QtGui.QColor(255, 204, 0), 1,
                         QtCore.Qt.DashLine)
    ]

    def __init__(self, subscription_id, driver_addr, sig_name, y_axis,
                 color_idx):
        """
        Initializes an instance of class CurveItem.

        driver_addr - IcePAP driver address.
        sig_name    - Signal name.
        y_axis      - Y axis to plot against.
        """
        self.subscription_id = subscription_id
        self.driver_addr = driver_addr
        self.signal_name = sig_name
        self.y_axis = y_axis
        self.array_time = []
        self.array_val = []
        self.val_min = 0
        self.val_max = 0
        col_item = self.colors[color_idx]
        self.color = col_item.pen_color
        self.pen = {'color': col_item.pen_color,
                    'width': col_item.pen_width,
                    'style': col_item.pen_style}
        self.curve = None
        self.lock = RLock()
        self.signature = ''
        self.update_signature()

    def update_signature(self):
        """Sets the new value of the signature string."""
        self.signature = '{}:{}:{}'.format(self.driver_addr,
                                           self.signal_name,
                                           self.y_axis)

    def create_curve(self):
        """Creates a new plot item."""
        with self.lock:
            self.curve = PlotCurveItem(x=self.array_time,
                                       y=self.array_val,
                                       pen=self.pen)
        return self.curve

    def update_curve(self, time_min, time_max):
        """Updates the curve with recent collected data."""
        with self.lock:
            idx_min = self.get_time_index(time_min)
            idx_max = self.get_time_index(time_max)
            self.curve.setData(x=self.array_time[idx_min:idx_max],
                               y=self.array_val[idx_min:idx_max])

    def in_range(self, t):
        """
        Check to see if time is within range of collected data.

        t - Time value.
        Return: True if time is within range of collected data.
                Otherwise False.
        """
        with self.lock:
            if self.array_time and \
                    self.array_time[0] < t < self.array_time[-1]:
                return True
        return False

    def start_time(self):
        """
        Get time for first data sample.

        Return: Time of the first collected data sample. -1 if none.
        """
        with self.lock:
            if self.array_time:
                return self.array_time[0]
        return -1

    def collect(self, new_data):
        """Store new collected data."""
        with self.lock:
            if not self.array_val:
                self.val_min = self.val_max = new_data[0][1]
            for t, v in new_data:
                self.array_time.append(t)
                self.array_val.append(v)
                if v > self.val_max:
                    self.val_max = v
                elif v < self.val_min:
                    self.val_min = v

    def get_y(self, time_val):
        """
        Retrieve the signal value corresponding to the provided time value.

        t_val - Time value.
        Return: Signal value corresponding to an adjacent sample in time.
        """
        with self.lock:
            idx = self.get_time_index(time_val)
            return self.array_val[idx]

    def clear(self):
        self.array_time[:] = []
        self.array_val[:] = []

    def get_time_index(self, time_val):
        """
        Retrieve the sample index corresponding to the provided time value.

        t_val - Time value.
        Return: Index of a sample adjacent to the provided time value.
        """
        with self.lock:
            if not self.array_time:
                return -1
            if len(self.array_time) == 1:
                return 0
            time_min = self.array_time[0]
            time_max = self.array_time[-1]
            if time_val < time_min:
                return 0
            elif time_val > time_max:
                return len(self.array_time)
            delta_t = time_max - time_min
            t = time_val - time_min
            idx = int((t / delta_t) * len(self.array_time))
            while self.array_time[idx] > time_val:
                idx -= 1
            while self.array_time[idx] < time_val:
                idx += 1
            return idx
